PYTHONPATH_ENV_VAR: str = "PYTHONPATH"
PHI_RUNTIME_ENV_VAR: str = "PHI_RUNTIME"

SCRIPTS_DIR_ENV_VAR: str = "PHI_SCRIPTS_DIR"
STORAGE_DIR_ENV_VAR: str = "PHI_STORAGE_DIR"
WORKFLOWS_DIR_ENV_VAR: str = "PHI_WORKFLOWS_DIR"
WORKSPACE_NAME_ENV_VAR: str = "PHI_WORKSPACE_NAME"
WORKSPACE_ROOT_ENV_VAR: str = "PHI_WORKSPACE_ROOT"
WORKSPACES_MOUNT_ENV_VAR: str = "PHI_WORKSPACES_MOUNT"
WORKSPACE_ID_ENV_VAR: str = "PHI_WORKSPACE_ID"
WORKSPACE_HASH_ENV_VAR: str = "PHI_WORKSPACE_HASH"
WORKSPACE_DIR_ENV_VAR: str = "PHI_WORKSPACE_DIR"
REQUIREMENTS_FILE_PATH_ENV_VAR: str = "REQUIREMENTS_FILE_PATH"

AWS_REGION_ENV_VAR: str = "AWS_REGION"
AWS_DEFAULT_REGION_ENV_VAR: str = "AWS_DEFAULT_REGION"
AWS_PROFILE_ENV_VAR: str = "AWS_PROFILE"
AWS_CONFIG_FILE_ENV_VAR: str = "AWS_CONFIG_FILE"
AWS_SHARED_CREDENTIALS_FILE_ENV_VAR: str = "AWS_SHARED_CREDENTIALS_FILE"

INIT_AIRFLOW_ENV_VAR: str = "INIT_AIRFLOW"
AIRFLOW_ENV_ENV_VAR: str = "AIRFLOW_ENV"
AIRFLOW_HOME_ENV_VAR: str = "AIRFLOW_HOME"
AIRFLOW_EXECUTOR_ENV_VAR: str = "AIRFLOW__CORE__EXECUTOR"
AIRFLOW_DAGS_FOLDER_ENV_VAR: str = "AIRFLOW__CORE__DAGS_FOLDER"
AIRFLOW_DB_CONN_URL_ENV_VAR: str = "AIRFLOW__DATABASE__SQL_ALCHEMY_CONN"
